use std::fs;
use std::path::Path;

use lindera_core::character_definition::CharacterDefinitions;
use lindera_core::connection::ConnectionCostMatrix;
use lindera_core::error::LinderaErrorKind;
use lindera_core::prefix_dict::PrefixDict;
use lindera_core::unknown_dictionary::UnknownDictionary;
use lindera_core::LinderaResult;

fn read_file(path: &Path) -> LinderaResult<Vec<u8>> {
    fs::read(path).map_err(|err| {
        LinderaErrorKind::Io.with_error(anyhow::anyhow!(
            "Failed to read {:?}, error: {:?}",
            path,
            err
        ))
    })
}

pub fn char_def(dir: &Path) -> LinderaResult<CharacterDefinitions> {
    let path = dir.join("char_def.bin");
    let data = read_file(&path)?;

    CharacterDefinitions::load(data.as_slice())
}

pub fn connection(dir: &Path) -> LinderaResult<ConnectionCostMatrix> {
    let path = dir.join("matrix.mtx");
    let data = read_file(&path)?;

    Ok(ConnectionCostMatrix::load(data.as_slice()))
}

pub fn prefix_dict(dir: &Path) -> LinderaResult<PrefixDict> {
    let unidic_data_path = dir.join("dict.da");
    let unidic_data = read_file(&unidic_data_path)?;

    let unidic_vals_path = dir.join("dict.vals");
    let unidic_vals = read_file(&unidic_vals_path)?;

    Ok(PrefixDict::from_static_slice(
        unidic_data.as_slice(),
        unidic_vals.as_slice(),
    ))
}

pub fn unknown_dict(dir: &Path) -> LinderaResult<UnknownDictionary> {
    let path = dir.join("unk.bin");
    let data = read_file(&path)?;

    UnknownDictionary::load(data.as_slice())
}

pub fn words_idx_data(dir: &Path) -> LinderaResult<Vec<u8>> {
    let path = dir.join("dict.wordsidx");
    read_file(&path)
}

pub fn words_data(dir: &Path) -> LinderaResult<Vec<u8>> {
    let path = dir.join("dict.words");
    read_file(&path)
}
